// robsound.cpp: Implementierung der Klasse SoundPlugin.
//
//////////////////////////////////////////////////////////////////////

#include "robsound.h"

#include <rtsound.h>

using namespace lrt;

namespace rt {

FpInterpretePos SoundPlugin::getInterpretePos()
{
	return fpInterPosBefore;
}

String SoundPlugin::getName()
{
	return "Sound Plugin";
}

String SoundPlugin::getHelpText()
{
	if(!Sound::isSupported())
		return "";
	
	return "  -sound     Enable sound output.";
}

bool SoundPlugin::canPlay(bool checkSilent)
{
	if(!Sound::isSupported())
	{
		if(!checkSilent)
			parent->handleWarning("Cannot play any sounds.");
		return false;
	}

	if(!isSoundInstalled("tick", checkSilent)
	|| !isSoundInstalled("win", checkSilent)
	|| !isSoundInstalled("loose", checkSilent)
	|| !isSoundInstalled("tie", checkSilent)
	|| !isSoundInstalled("end", checkSilent))
		return false;

	return true;
}

bool SoundPlugin::interpreteParams(const Array<String>& params, Array<bool>& used)
{
	for(int i = 0; i < params.length(); i++)
		if(params[i].lowerCase() == "-sound") // Yippee! We may play! ;-)
		{
			active = true;
			used[i] = true;
		}
	return true;
}

void SoundPlugin::fillOptions(SimOptions& options)
{
	if(!active) return;

	if(!canPlay(false)) // really play?
	{
		active = false;
		return;
	}

	options.tourDisps += new SoundTourDisplayer();
	options.supervisors += new SoundSimSupervisor();
}

bool SoundPlugin::isSoundInstalled(const String& sound, bool checkSilent)
{
	bool inst = Sound::isPresent(sound);
	if(!checkSilent && !inst)
		parent->handleWarning("Required sound not found: " + sound + Sound::getExtension());
	return inst;
}


SoundPlugin::SoundPlugin(Frontend* parent) : FrontendPlugin(parent)
{
}

//////////////////////// SoundTourDisplayer //////////////////////////////////

void SoundTourDisplayer::init(const TourInitInfo& info)
{}

TourStatusType SoundTourDisplayer::update(const Array<TourResult>& status)
{ return tourContinue; }

void SoundTourDisplayer::exit(TourStatusType exitType, const Array<TourResult>& result)
{
	Sound::play("end", true);
}


///////////////////////// SoundSimSupervisor

const int SoundSimSupervisor::minMillis = 60000 / 320;	// 200 BPM
const int SoundSimSupervisor::maxMillis = 60000 / 80;		// 50 BPM


SoundSimSupervisor::SoundSimSupervisor() : cycles(1000), lastTime(Time::getCurrentTime())
{
	retState.affectedProgram = 0;
	retState.event = gameNothing;
}

SimSuperPosition SoundSimSupervisor::getPreferredPosition()
{
	return supPosDontCare;
}

unsigned long SoundSimSupervisor::getStepCycles()
{
	return cycles;
}

SimSupervisor::GameState SoundSimSupervisor::exec(Simulation* const curSim)
{
	Time curTime = Time::getCurrentTime();
	int diff = (curTime - lastTime).toInt();
	lastTime = curTime;

	if(curSim->curCycle <= 1) return retState; // do nothing at sim begin

	if((diff < minMillis) && (diff > 2))
		cycles *= 2;
	else if(diff > maxMillis)
		cycles /= 2;

	Sound::play("tick", false, false);

	return retState;
}

void SoundSimSupervisor::exitSim(Simulation* const curSim, const GameState& simResult)
{
	if(simResult.event == gameFinished)
	{
		if(simResult.affectedProgram == 0)
			Sound::play("win", false);
		else
			Sound::play("loose", false);
	}
	else if(simResult.event == gameTie)
		Sound::play("tie", false);
}



} // namespace

